;;; AssyMass calulates and shows the mass and centre of gravity of an assembly on the screen
;;;
;;;   There are options below for steel and aluminium. You need to rem out the line that is not required.
;;;

(setq PDen 7.850 matltxt "Steel")          ;;;Steel Density
;(setq PDen 2.710 matltxt "Aluminium")      ;;;Aluminium Density


(defun c:AssyMass (/ tolist centroid
                       overallcentroid ss1
                       sumofmomemnts sumofmoments totalvolume
                       volume cmdecho *ERROR*
                    )
  (vl-load-com)
  (defun ToList (col / rtlist)
    (cond
      ((listp col) col)
      ((= 'STR (type col)) (vl-string->list col))
      ((= 'PICKSET (type col))
          (vl-remove-if
            'null
            (vl-remove-if-not
              (function (lambda (e) (= 'ENAME (type e))))
              (apply 'append (ssnamex col))
            )
          )
      )
      ((and (= 'variant (type col))
            (<= 8192 (vlax-variant-type col))
       )
        (vlax-safearray->list (vlax-variant-value col))
      )
      ((not (vl-catch-all-error-p
              (vl-catch-all-apply 'vla-get-count (list col))
            )
       )
        (progn
          (vlax-for item col (setq rtlist (cons item rtlist)))
          (reverse rtlist)
        )
      )
    )
  )
  (defun *ERROR* (msg /)
    (if cmdecho (setvar "cmdecho" cmdecho))
    (princ msg)
  )
  (setq cmdecho (getvar "cmdecho"))
  (setvar "cmdecho" 0)
  (cond
    ((vl-catch-all-error-p
       (setq ss1 (vl-catch-all-apply 'ssget '(((0 . "3DSOLID")))))
     )
      (princ (strcat "\nError|" (vl-catch-all-error-message ss1)))
    )
    ((= 0 (sslength ss1))
        nil
    )
    (T
      (setq
        SumOfMoments '(0 0 0)
        TotalVolume 0
      )
      (princ "\n")
      (foreach obj (mapcar (function (lambda (e) (vlax-ename->vla-object e)))
                           (tolist ss1)
                   )
        (prompt ".")
        (setq
          Volume (vla-get-volume obj)
          Centroid (vlax-safearray->list (vlax-variant-value (vla-get-centroid obj)))
          SumOfMoments (list
                         (+ (car SumOfMoments) (* Volume (car Centroid)))
                         (+ (cadr SumOfMoments) (* Volume (cadr Centroid)))
                         (+ (caddr SumOfMoments) (* Volume (caddr Centroid)))
                       )
          TotalVolume (+ TotalVolume Volume)
        )
      )
      (setq
        OverallCentroid
        (mapcar (function (lambda (ord) (/ ord TotalVolume)))
                SumOfMoments
        )
      )
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
;;;
;;;                            Draw a circle at the centriod
;;;
;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;;
      (command "_Circle"
               "_non"
               (trans OverallCentroid 0 1)
               "d"
               (getvar "viewsize")
      )
      (setq cog_loc
        (strcat "\nCircle drawn at the centroid of the selected solids ("
                (vl-prin1-to-string (trans OverallCentroid 0 1))
                ")"
        )
      )
      (setq tot_vol (strcat "\nTotal Volume: "
                      (rtos (* totalvolume 0.000000001) 2 3)
                      " cubic metres"
              )
      )
      (setq tot_mas
        (strcat "\n" matltxt " weight: "
                (rtos (* PDen (/ totalvolume 1000 1000 )) 2 2)
                " kgs."
        )
      )
      (setq mod_stat (strcat tot_vol "\n" tot_mas))
      (LM:popup "Assembly Mass" mod_stat 0)
    )
  )
  (if cmdecho (setvar "cmdecho" cmdecho))
)
(defun LM:popup ( ttl msg bit / wsh rtn )
    (if (setq wsh (vlax-create-object "wscript.shell"))
        (progn
            (setq rtn (vl-catch-all-apply 'vlax-invoke-method (list wsh 'popup msg 0 ttl bit)))
            (vlax-release-object wsh)
            (if (not (vl-catch-all-error-p rtn)) rtn)
        )
    )
)
